package com.nominanuda.hyperapi.async;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.FutureTask;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import com.nominanuda.lang.Check;

public class CopyOfAsyncInvokerImpl implements AsyncInvoker {
	private ExecutorService threadPool;
	private List<FutureTask<?>> futures = new LinkedList<FutureTask<?>>();

	public <T,Z> Z async(final Z delegee, final SuccessCallback<T> ss, final ErrorCallback ee) {
		InvocationHandler handler = new InvocationHandler() {
			public Object invoke(final Object proxy, final Method m, final Object[] args)
					throws Throwable {
				Callable<T> task = new Callable<T>() {
					public T call() throws Exception {
						try {
							T res = (T)m.invoke(delegee, args);
							ss.apply(res, "textStatus", null);
							return res;
						} catch(InvocationTargetException e) {
							ee.apply(null, "textStatus", (Exception)e.getTargetException());
							return null;
						}
					}
				};
				FutureTask<T> fut = enqueueTask(task);
				futures.add(fut);
				return null;
			}
		};
		Object proxy = Proxy.newProxyInstance(
			getClass().getClassLoader(), delegee.getClass().getInterfaces(), handler);
		return (Z)proxy;
	}

	protected <T> FutureTask<T> enqueueTask(Callable<T> task) {
		FutureTask<T> fut = new FutureTask<T>(task);
		getThreadPool().submit(fut);
		return fut;
	}
	public void setThreadPool(ExecutorService threadPool) {
		this.threadPool = threadPool;
	}


	protected ExecutorService getThreadPool() {
		if(threadPool == null) {
			threadPool = Executors.newCachedThreadPool();
		}
		return threadPool;
	}

	public List<FutureTask<?>> getFutures() {
		return futures;
	}

	public void await(long timeout, TimeUnit unit) {
		long start = System.currentTimeMillis();
		long millis = TimeUnit.MILLISECONDS.convert(timeout, unit);
		for(Future<?> f : futures) {
			if(System.currentTimeMillis() - start > millis) {
				break;//TODO throw ?
			}
			try {
				f.get(timeout, unit);
			} catch (InterruptedException e) {
			} catch (ExecutionException e) {
			} catch (TimeoutException e) {
			}
		}
	}

}
